﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using MongoDB;


/// <summary>
/// 操作“订单记录”的业务逻辑层
/// </summary>
public sealed class OrderBLL 
{
	// 这里演示了MongoDB在【表关联】的现实使用场景。
	// 由于MongoDB并不支持 join 这样的操作，它更提倡将对象完整保存，
	// 所以，要想实现【表关联】或者【记录引用】这类操作，只好分成多个查询步骤来处理。

	// 虽然MongoDB支持所谓的【DBRef】，但发现使用起来，有些繁琐，而且一次只能返回一个Ref对象，
	// 这对于：【需要返回一个列表】这样的使用场景来说，会导致更多的数据库查询，效率也不高。
	// 因此本示例将不使用这个功能，而是直接采用【保存ID手工处理查询】的方式来处理这类引用查询。
	// 幸好，我们有LINQ，处理起来也不是太难。 :)

	// 如果您有更好的解决办法，那么请告诉我，谢谢了。


	/// <summary>
	/// 数据库中，订单集合的名称
	/// </summary>
	public static readonly string STR_Orders = "Orders";


	/// <summary>
	/// 新增订单记录（包含订单明细）
	/// </summary>
	/// <param name="order"></param>
	/// <returns></returns>
	public string AddOrder(OrderItem order)
	{
		// 为订单分配一个ID
		order.OrderID = Guid.NewGuid().ToString("N");

		// 先获取所有订单明细的商品ID
		string[] pidArray = order.Detail.Select(x => x.ProductID).ToArray();

		using( MyMongoDb mm = new MyMongoDb() ) {
			var collection = mm.GetCollection<OrderItem>(STR_Orders);

			// 获取所有订单记录对应的商品记录：订单在保存时需要知道商品价格。
			var products = mm.GetCollection<Product>().Linq()
								.Where(p => pidArray.Contains(p.ProductID))
								.ToDictionary(x => x.ProductID);

			// 为每个订单明细记录设置【商品属性】，累加订单总金额。
			for( int i = order.Detail.Count - 1; i >= 0; i-- ) {
				OrderDetail detail = order.Detail.ElementAt(i);

				Product p = null;
				if( products.TryGetValue(detail.ProductID, out p) ) {
					detail.UnitPrice = p.UnitPrice;
					detail.ProductName = p.ProductName;
					detail.Unit = p.Unit;
					order.SumMoney += detail.UnitPrice * detail.Quantity;
				}
				else {
					order.Detail.Remove(detail);
				}
			}

			// 将【订单的主记录和明细记录】一起保存到数据库。
			// 因为MongoDB不支持事务，所以这样做是安全的。
			collection.Insert(order);
		}

		return order.OrderID;
	}


	/// <summary>
	/// 根据指定的查询日期范围及分页参数，获取订单记录列表
	/// </summary>
	/// <param name="option">日期范围及分页参数</param>
	/// <returns>订单记录列表</returns>
	public List<OrderItem> Search(OrderSearchOption option)
	{
		option.EndDate = option.EndDate.AddDays(1);

		using( MyMongoDb mm = new MyMongoDb() ) {
			var collection = mm.GetCollection<OrderItem>(STR_Orders);

			var query = from ord in collection.Linq()
						where ord.OrderDate >= option.StartDate && ord.OrderDate < option.EndDate
						orderby ord.OrderDate descending
						select new OrderItem {
							OrderID = ord.OrderID,
							CustomerID = ord.CustomerID,
							OrderDate = ord.OrderDate,
							SumMoney = ord.SumMoney,
							Finished = ord.Finished
						};

			// 获取订单列表，此时将返回符合分页的结果。
			List<OrderItem> list = query.GetPagingList<OrderItem>(option);


			// 获取订单列表中所有的客户ID
			string[] cids = (from ord in list 
							 where string.IsNullOrEmpty(ord.CustomerID) == false
							 select ord.CustomerID)
							 .Distinct()
							 .ToArray();

			// 找到所有客户记录
			Dictionary<string, Customer> customers =
								(from c in mm.GetCollection<Customer>().Linq()
								 where cids.Contains(c.CustomerID)
								 select new Customer {
									 CustomerID = c.CustomerID,
									 CustomerName = c.CustomerName
								 })
								.ToDictionary(x => x.CustomerID);

			// 为订单列表结果设置CustomerName
			foreach( OrderItem ord in list ) {
				Customer c = null;
				if( string.IsNullOrEmpty(ord.CustomerID) == false && customers.TryGetValue(ord.CustomerID, out c) )
					ord.CustomerName = c.CustomerName;
				else 
					ord.CustomerName = string.Empty;
			}

			return list;
		}
	}

	/// <summary>
	/// 根据订单ID获取订单相关的所有信息
	/// </summary>
	/// <param name="orderId"></param>
	/// <returns></returns>
	public OrderItem GetOrderById(string orderId)
	{
		using( MyMongoDb mm = new MyMongoDb() ) {
			var collection = mm.GetCollection<OrderItem>(STR_Orders);

			// 获取一条订单记录，此时将包含它的明细记录。
			OrderItem ord = collection.FindOne(x => x.OrderID == orderId);

			if( ord != null ) {
				ord.CustomerName = string.Empty;

				if( string.IsNullOrEmpty(ord.CustomerID) == false ) {
					// 获取订单的客户名称
					Customer customer = mm.GetCollection<Customer>().Linq()
											.FirstOrDefault(x => x.CustomerID == ord.CustomerID);
					if( customer != null )
						ord.CustomerName = customer.CustomerName;
				}
			}
			return ord;
		}
	}

	/// <summary>
	/// 修改指定的订单状态
	/// </summary>
	/// <param name="orderId"></param>
	/// <param name="finished"></param>
	/// <returns></returns>
	public void SetOrderStatus(string orderId, bool finished)
	{
		using( MyMongoDb mm = new MyMongoDb() ) {
			var collection = mm.GetCollection<OrderItem>(STR_Orders);

			// 根据订单ID获取对应的记录
			OrderItem order = collection.FindOne(x => x.OrderID == orderId);
			if( order == null )
				return;

			// 设置完成标记
			order.Finished = finished;

			// 更新数据库。
			collection.Update(order, (x => x.OrderID == orderId));
		}
	}



}


