﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Script.Serialization;
using MongoDB.Attributes;


// 说明：因为MonogDB并不支持【自增数据类型】，所以，我把所有实体的ID字段的类型全改为字符串，它们将保存GUID字符串。

// 另外，这里直接在订单明细中保存了ProductName，Unit 。主要是为了减少与数据库的交互次数（事实上这样做也合理）。




/// <summary>
/// 表示一个订单头记录
/// </summary>
public sealed class OrderItem : MyDataItem
{
	[MongoId]		// 这个成员将映射到 "_id"
	public string OrderID { get; set; }
	[ScriptIgnore]
	public string CustomerID { get; set; }
	[ScriptIgnore]	// 在JSON序列化时，忽略这个成员
	public DateTime OrderDate { get; set; }

	public double SumMoney { get; set; }

	public string Comment { get; set; }

	public bool Finished { get; set; }

	[MongoIgnore]	// 在保存到MongoDB时，忽略这个成员
	public string CustomerName { get; set; }

	public List<OrderDetail> Detail;

	public string OrderNo
	{
		get
		{
			// 以前的DEMO是根据ID采用前缀补0计算出来的。
			if( this.OrderID.Length < 12 )
				return this.OrderID.PadLeft(12, '0');
			else
				// 这个演示中直接使用GUID
				return this.OrderID;
		}
	}
	public string ValidCustomerId
	{
		get { return this.CustomerID ?? string.Empty; }
	}

	// 加这个属性仅仅为了在客户端中能更容易的显示，要不然，客户端处理格式转换实在是麻烦。
	// 它将不会被写入到数据库。
	[MongoIgnore]
	public string OrderDateText { get { return this.OrderDate.ToString("yyyy-MM-dd HH:mm:ss"); } }

}

/// <summary>
/// 订单明细
/// </summary>
public sealed class OrderDetail : MyDataItem
{
	//[MongoId]
	//public string OrderDetailID { get; set; }
	public string OrderID { get; set; }
	public string ProductID { get; set; }
	public double UnitPrice { get; set; }
	public int Quantity { get; set; }

	//[MongoIgnore]
	public string ProductName { get; set; }
	//[MongoIgnore]
	public string Unit { get; set; }

}


/// <summary>
/// 页面提交订单的数据对象
/// </summary>
public sealed class OrderSubmitForm : MyDataItem
{
	public DateTime OrderDate;
	public string CustomerID;
	public string OrderDetail;
	public string Comment;
	

	public override string IsValid()
	{
		if( this.OrderDate.Year < 2000 )
			return "无效的日期。";

		if( string.IsNullOrEmpty(OrderDetail) )
			return "没有订单明细项目。";

		return null;
	}

	public OrderItem ConvertToOrderItem()
	{
		// 验证对象是否有效
		string error = this.IsValid();
		if( string.IsNullOrEmpty(error) == false )
			throw new MyMessageException(error);

		// 创建实体对象
		OrderItem order = new OrderItem();
		order.OrderDate = this.OrderDate.PadTime();
		if( string.IsNullOrEmpty( this.CustomerID) == false )
			order.CustomerID = this.CustomerID;
		order.Comment = this.Comment;

		order.Detail = new List<OrderDetail>();
		// 将 id1=2;id2=3; 的格式字符串分解出来
		List<KeyValuePair<string, string>> detail = this.OrderDetail.SplitString(';', '=');
		// 将列表转成订单明细
		detail.ForEach(x => order.Detail.Add(new OrderDetail { ProductID = x.Key, Quantity = int.Parse(x.Value) }));

		return order;
	}
}
